@@ -21,12 +21,12 @@ from typing import Optional
 from qtpy import PYQT5
 from qtpy.QtCore import QByteArray, QSize, Qt, Signal, Slot
 from qtpy.QtGui import QIcon
-from qtpy.QtWidgets import (QHBoxLayout, QSizePolicy, QToolButton, QVBoxLayout,
-                            QWidget)
+from qtpy.QtWidgets import (QApplication, QHBoxLayout, QSizePolicy,
+                            QToolButton, QVBoxLayout, QWidget)
 
 # Local imports
 from spyder.api.exceptions import SpyderAPIError
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.api.widgets.auxiliary_widgets import (MainCornerWidget,
                                                   SpyderWindowWidget)
 from spyder.api.widgets.menus import (MainWidgetMenu, OptionsMenuSections,
@@ -39,11 +39,9 @@ from spyder.utils.registries import (
     ACTION_REGISTRY, TOOLBAR_REGISTRY, MENU_REGISTRY)
 from spyder.utils.stylesheet import (
     APP_STYLESHEET, PANES_TABBAR_STYLESHEET, PANES_TOOLBAR_STYLESHEET)
-from spyder.widgets.dock import SpyderDockWidget
+from spyder.widgets.dock import DockTitleBar, SpyderDockWidget
 from spyder.widgets.tabs import Tabs
 
-# Localization
-_ = get_translation('spyder')
 
 # Logging
 logger = logging.getLogger(__name__)
@@ -60,6 +58,7 @@ class PluginMainWidgetActions:
     ClosePane = 'close_pane'
     DockPane = 'dock_pane'
     UndockPane = 'undock_pane'
+    LockUnlockPosition = 'lock_unlock_position'
 
 
 class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
@@ -82,8 +81,8 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
     stacked vertically and cannot be placed horizontally next to each other.
     """
 
-    # --- Attributes
-    # ------------------------------------------------------------------------
+    # ---- Attributes
+    # -------------------------------------------------------------------------
     ENABLE_SPINNER = False
     """
     This attribute enables/disables showing a spinner on the top right to the
@@ -113,8 +112,8 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
     the plugin, then this attribute should have a `None` value.
     """
 
-    # --- Signals
-    # ------------------------------------------------------------------------
+    # ---- Signals
+    # -------------------------------------------------------------------------
     sig_free_memory_requested = Signal()
     """
     This signal can be emitted to request the main application to garbage
@@ -173,7 +172,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
 
     sig_toggle_view_changed = Signal(bool)
     """
-    This action is emitted to inform the visibility of a dockable plugin
+    This signal is emitted to inform the visibility of a dockable plugin
     has changed.
 
     This is triggered by checking/unchecking the entry for a pane in the
@@ -191,6 +190,17 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
     needs its ancestor to be updated.
     """
 
+    sig_unmaximize_plugin_requested = Signal((), (object,))
+    """
+    This signal is emitted to inform the main window that it needs to
+    unmaximize the currently maximized plugin, if any.
+
+    Parameters
+    ----------
+    plugin_instance: SpyderDockablePlugin
+        Unmaximize plugin only if it is not `plugin_instance`.
+    """
+
     def __init__(self, name, plugin, parent=None):
         if PYQT5:
             super().__init__(parent=parent, class_parent=plugin)
@@ -205,12 +215,12 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         self._plugin = plugin
         self._parent = parent
         self._default_margins = None
-        self.is_maximized = None
         self.is_visible = None
         self.dock_action = None
         self.undock_action = None
         self.close_action = None
         self._toolbars_already_rendered = False
+        self._is_maximized = False
 
         # Attribute used to access the action, toolbar, toolbutton and menu
         # registries
@@ -290,8 +300,8 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         self._toolbars_layout.addLayout(self._main_toolbar_layout)
         self._main_layout.addLayout(self._toolbars_layout, stretch=1)
 
-    # --- Private Methods
-    # ------------------------------------------------------------------------
+    # ---- Private Methods
+    # -------------------------------------------------------------------------
     def _setup(self):
         """
         Setup default actions, create options menu, and connect signals.
@@ -338,6 +348,13 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
             icon=self.create_icon('dock'),
             triggered=self.close_window,
         )
+        self.lock_unlock_action = self.create_action(
+            name=PluginMainWidgetActions.LockUnlockPosition,
+            text=_("Unlock position"),
+            tip=_("Unlock to move pane to another position"),
+            icon=self.create_icon('drag_dock_widget'),
+            triggered=self.lock_unlock_position,
+        )
         self.undock_action = self.create_action(
             name=PluginMainWidgetActions.UndockPane,
             text=_("Undock"),
@@ -361,12 +378,12 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
             shortcut_context='_',
         )
 
-        bottom_section = OptionsMenuSections.Bottom
-        for item in [self.undock_action, self.close_action, self.dock_action]:
+        for item in [self.lock_unlock_action, self.undock_action,
+                     self.close_action, self.dock_action]:
             self.add_item_to_menu(
                 item,
                 self._options_menu,
-                section=bottom_section,
+                section=OptionsMenuSections.Bottom,
             )
 
         self._options_button.setMenu(self._options_menu)
@@ -397,6 +414,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         show_dock_actions = self.windowwidget is None
         self.undock_action.setVisible(show_dock_actions)
         self.close_action.setVisible(show_dock_actions)
+        self.lock_unlock_action.setVisible(show_dock_actions)
         self.dock_action.setVisible(not show_dock_actions)
 
         if sys.platform == 'darwin':
@@ -418,8 +436,33 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         """
         self.undock_action.setDisabled(top_level)
 
-    # --- Public Qt overriden methods
-    # ------------------------------------------------------------------------
+        # Change the cursor shape when dragging
+        if top_level:
+            QApplication.setOverrideCursor(Qt.ClosedHandCursor)
+        else:
+            QApplication.restoreOverrideCursor()
+
+    @Slot(bool)
+    def _on_title_bar_shown(self, visible):
+        """
+        Actions to perform when the title bar is shown/hidden.
+        """
+        if visible:
+            self.lock_unlock_action.setText(_('Lock position'))
+            self.lock_unlock_action.setIcon(self.create_icon('lock_open'))
+            for method_name in ['setToolTip', 'setStatusTip']:
+                method = getattr(self.lock_unlock_action, method_name)
+                method(_("Lock pane to the current position"))
+        else:
+            self.lock_unlock_action.setText(_('Unlock position'))
+            self.lock_unlock_action.setIcon(
+                self.create_icon('drag_dock_widget'))
+            for method_name in ['setToolTip', 'setStatusTip']:
+                method = getattr(self.lock_unlock_action, method_name)
+                method(_("Unlock to move pane to another position"))
+
+    # ---- Public Qt overriden methods
+    # -------------------------------------------------------------------------
     def setLayout(self, layout):
         """
         Set layout of the main widget of this plugin.
@@ -433,8 +476,8 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         self.on_close()
         super().closeEvent(event)
 
-    # --- Public methods to use
-    # ------------------------------------------------------------------------
+    # ---- Public methods to use
+    # -------------------------------------------------------------------------
     def get_plugin(self):
         """
         Return the parent plugin.
@@ -701,8 +744,8 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
 
             # self._toolbars_already_rendered = True
 
-    # --- SpyderDockwidget handling ------------------------------------------
-    # ------------------------------------------------------------------------
+    # ---- SpyderDockwidget handling ------------------------------------------
+    # -------------------------------------------------------------------------
     @Slot()
     def create_window(self):
         """
@@ -743,9 +786,18 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         window.show()
 
     @Slot()
-    def close_window(self):
+    def close_window(self, save_undocked=False):
         """
         Close QMainWindow instance that contains this widget.
+
+        Parameters
+        ----------
+        save_undocked : bool, optional
+            True if the undocked state needs to be saved. The default is False.
+
+        Returns
+        -------
+        None.
         """
         logger.debug("Docking plugin back to the main window")
 
@@ -755,6 +807,10 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
             geometry = self.windowwidget.saveGeometry()
             self.set_conf('window_geometry', qbytearray_to_str(geometry))
 
+            # Save undocking state if requested
+            if save_undocked:
+                self.set_conf('undocked_on_window_close', True)
+
             # Fixes spyder-ide/spyder#10704
             self.__unsafe_window = self.windowwidget
             self.__unsafe_window.deleteLater()
@@ -767,10 +823,14 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
 
             if self.dockwidget is not None:
                 self.sig_update_ancestor_requested.emit()
+                self.get_plugin().switch_to_plugin()
                 self.dockwidget.setWidget(self)
                 self.dockwidget.setVisible(True)
                 self.dockwidget.raise_()
                 self._update_actions()
+        else:
+            # Reset undocked state
+            self.set_conf('undocked_on_window_close', False)
 
     def change_visibility(self, enable, force_focus=None):
         """Dock widget visibility has changed."""
@@ -823,6 +883,10 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         if not self.dockwidget:
             return
 
+        # Dock plugin if it's undocked before hiding it.
+        if self.windowwidget is not None:
+            self.close_window(save_undocked=True)
+
         if checked:
             self.dockwidget.show()
             self.dockwidget.raise_()
@@ -854,6 +918,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         # Signals
         dock.visibilityChanged.connect(self.change_visibility)
         dock.topLevelChanged.connect(self._on_top_level_change)
+        dock.sig_title_bar_shown.connect(self._on_title_bar_shown)
 
         return (dock, dock.LOCATION)
 
@@ -864,6 +929,33 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         """
         self.toggle_view(False)
 
+    def lock_unlock_position(self):
+        """
+        Show/hide title bar to move/lock position.
+        """
+        if isinstance(self.dockwidget.titleBarWidget(), DockTitleBar):
+            self.dockwidget.remove_title_bar()
+        else:
+            self.dockwidget.set_title_bar()
+
+    def get_maximized_state(self):
+        """Get dockwidget's maximized state."""
+        return self._is_maximized
+
+    def set_maximized_state(self, state):
+        """
+        Set internal attribute that holds dockwidget's maximized state.
+
+        Parameters
+        ----------
+        state: bool
+            True if the plugin is maximized, False otherwise.
+        """
+        self._is_maximized = state
+
+        # This is necessary for old API plugins interacting with new ones.
+        self._plugin._ismaximized = state
+
     # --- API: methods to define or override
     # ------------------------------------------------------------------------
     def get_title(self):
@@ -903,6 +995,7 @@ class PluginMainWidget(QWidget, SpyderWidgetMixin, SpyderToolbarMixin):
         """
         pass
 
+
 def run_test():
     # Third party imports
     from qtpy.QtWidgets import QHBoxLayout, QTableWidget, QMainWindow
