@@ -27,7 +27,7 @@ from qtpy.QtWidgets import QAction, QInputDialog, QLineEdit, QVBoxLayout
 # Local imports
 from spyder.api.exceptions import SpyderAPIError
 from spyder.api.plugin_registration.registry import PLUGIN_REGISTRY
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.api.widgets.main_widget import PluginMainWidget
 from spyder.api.config.decorators import on_conf_change
 from spyder.utils.installers import InstallerInternalError
@@ -42,8 +42,8 @@ from spyder.widgets.collectionseditor import CollectionsEditor
 from spyder.widgets.findreplace import FindReplace
 from spyder.widgets.reporterror import SpyderErrorDialog
 
-# Localization
-_ = get_translation('spyder')
+
+# Logging
 logger = logging.getLogger(__name__)
 
 
@@ -95,9 +95,6 @@ class ConsoleWidget(PluginMainWidget):
     # Request to show a status message on the main window
     sig_show_status_requested = Signal(str)
 
-    # Request the main application to quit.
-    sig_quit_requested = Signal()
-
     sig_help_requested = Signal(dict)
     """
     This signal is emitted to request help on a given object `name`.
@@ -152,6 +149,7 @@ class ConsoleWidget(PluginMainWidget):
 
         # Layout
         layout = QVBoxLayout()
+        layout.setSpacing(0)
         layout.addWidget(self.shell)
         layout.addWidget(self.find_widget)
         self.setLayout(layout)
@@ -188,7 +186,7 @@ class ConsoleWidget(PluginMainWidget):
         run_action = self.create_action(
             ConsoleWidgetActions.Run,
             text=_("&Run..."),
-            tip=_("Run a Python script"),
+            tip=_("Run a Python file"),
             icon=self.create_icon('run_small'),
             triggered=self.run_script,
         )
@@ -429,7 +427,6 @@ class ConsoleWidget(PluginMainWidget):
                 self.error_dlg = SpyderErrorDialog(self)
                 self.error_dlg.set_color_scheme(
                     self.get_conf('selected', section='appearance'))
-                self.error_dlg.close_btn.clicked.connect(self.close_error_dlg)
                 self.error_dlg.rejected.connect(self.remove_error_dlg)
                 self.error_dlg.details.sig_go_to_error_requested.connect(
                     self.go_to_error)
@@ -458,15 +455,17 @@ class ConsoleWidget(PluginMainWidget):
         """
         Close error dialog.
         """
-        if self.error_dlg.dismiss_box.isChecked():
-            self.dismiss_error = True
-
-        self.error_dlg.reject()
+        if self.error_dlg:
+            self.error_dlg.reject()
 
     def remove_error_dlg(self):
         """
         Remove error dialog.
         """
+        if self.error_dlg.dismiss_box.isChecked():
+            self.dismiss_error = True
+
+        self.error_dlg.disconnect()
         self.error_dlg = None
 
     @Slot()
@@ -506,9 +505,9 @@ class ConsoleWidget(PluginMainWidget):
             self.shell.interpreter.restore_stds()
             filename, _selfilter = getopenfilename(
                 self,
-                _("Run Python script"),
+                _("Run Python file"),
                 getcwd_or_home(),
-                _("Python scripts") + " (*.py ; *.pyw ; *.ipy)",
+                _("Python files") + " (*.py ; *.pyw ; *.ipy)",
             )
             self.shell.interpreter.redirect_stds()
 
