@@ -17,7 +17,7 @@ from qtpy.QtWidgets import QVBoxLayout, QWidget
 
 # Local imports
 from spyder.api.config.decorators import on_conf_change
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.api.widgets.main_widget import PluginMainWidget
 from spyder.py3compat import is_text_string, to_text_string
 from spyder.utils import encoding
@@ -27,9 +27,6 @@ from spyder.widgets.simplecodeeditor import SimpleCodeEditor
 from spyder.widgets.tabs import Tabs
 from spyder.utils.stylesheet import PANES_TABBAR_STYLESHEET
 
-# Localization
-_ = get_translation('spyder')
-
 
 # --- Constants
 # ----------------------------------------------------------------------------
@@ -86,6 +83,7 @@ class HistoryWidget(PluginMainWidget):
 
         # Layout
         layout = QVBoxLayout()
+        layout.setSpacing(0)
 
         # TODO: Move this to the tab container directly
         if sys.platform == 'darwin':
@@ -152,8 +150,11 @@ class HistoryWidget(PluginMainWidget):
         for editor in self.editors:
             editor.toggle_line_numbers(value)
 
-    @on_conf_change(option='selected', section='appearance')
-    def on_color_scheme_change(self, value):
+    @on_conf_change(section='appearance', option=['selected', 'ui_theme'])
+    def on_color_scheme_change(self, option, value):
+        if option == 'ui_theme':
+            value = self.get_conf('selected', section='appearance')
+
         for editor in self.editors:
             editor.set_color_scheme(value)
 
@@ -270,7 +271,7 @@ class HistoryWidget(PluginMainWidget):
         self.tabwidget.setTabToolTip(index, filename)
 
         # Signals
-        editor.sig_focus_changed.connect(lambda: self.sig_focus_changed.emit())
+        editor.sig_focus_changed.connect(self.sig_focus_changed)
 
     @Slot(str, str)
     def append_to_history(self, filename, command):
@@ -318,6 +319,10 @@ class HistoryWidget(PluginMainWidget):
             padding='4px'
         )
 
+        css['QTabWidget::pane'].setValues(
+            border='1px',
+        )
+
         css['QTabWidget::left-corner'].setValues(
             left='0px',
         )
